// Code generated by smithy-go-codegen DO NOT EDIT.

package lakeformation

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/lakeformation/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns the LF-Tag expressions in caller’s account filtered based on caller's
// permissions. Data Lake and read only admins implicitly can see all tag
// expressions in their account, else caller needs DESCRIBE permissions on tag
// expression.
func (c *Client) ListLFTagExpressions(ctx context.Context, params *ListLFTagExpressionsInput, optFns ...func(*Options)) (*ListLFTagExpressionsOutput, error) {
	if params == nil {
		params = &ListLFTagExpressionsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListLFTagExpressions", params, optFns, c.addOperationListLFTagExpressionsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListLFTagExpressionsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListLFTagExpressionsInput struct {

	// The identifier for the Data Catalog. By default, the account ID.
	CatalogId *string

	// The maximum number of results to return.
	MaxResults *int32

	// A continuation token, if this is not the first call to retrieve this list.
	NextToken *string

	noSmithyDocumentSerde
}

type ListLFTagExpressionsOutput struct {

	// Logical expressions composed of one more LF-Tag key-value pairs.
	LFTagExpressions []types.LFTagExpression

	// A continuation token, if this is not the first call to retrieve this list.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListLFTagExpressionsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListLFTagExpressions{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListLFTagExpressions{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListLFTagExpressions"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListLFTagExpressions(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addSpanInitializeStart(stack); err != nil {
		return err
	}
	if err = addSpanInitializeEnd(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestStart(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestEnd(stack); err != nil {
		return err
	}
	return nil
}

// ListLFTagExpressionsPaginatorOptions is the paginator options for
// ListLFTagExpressions
type ListLFTagExpressionsPaginatorOptions struct {
	// The maximum number of results to return.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListLFTagExpressionsPaginator is a paginator for ListLFTagExpressions
type ListLFTagExpressionsPaginator struct {
	options   ListLFTagExpressionsPaginatorOptions
	client    ListLFTagExpressionsAPIClient
	params    *ListLFTagExpressionsInput
	nextToken *string
	firstPage bool
}

// NewListLFTagExpressionsPaginator returns a new ListLFTagExpressionsPaginator
func NewListLFTagExpressionsPaginator(client ListLFTagExpressionsAPIClient, params *ListLFTagExpressionsInput, optFns ...func(*ListLFTagExpressionsPaginatorOptions)) *ListLFTagExpressionsPaginator {
	if params == nil {
		params = &ListLFTagExpressionsInput{}
	}

	options := ListLFTagExpressionsPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListLFTagExpressionsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListLFTagExpressionsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListLFTagExpressions page.
func (p *ListLFTagExpressionsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListLFTagExpressionsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	optFns = append([]func(*Options){
		addIsPaginatorUserAgent,
	}, optFns...)
	result, err := p.client.ListLFTagExpressions(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

// ListLFTagExpressionsAPIClient is a client that implements the
// ListLFTagExpressions operation.
type ListLFTagExpressionsAPIClient interface {
	ListLFTagExpressions(context.Context, *ListLFTagExpressionsInput, ...func(*Options)) (*ListLFTagExpressionsOutput, error)
}

var _ ListLFTagExpressionsAPIClient = (*Client)(nil)

func newServiceMetadataMiddleware_opListLFTagExpressions(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListLFTagExpressions",
	}
}
