// Code generated by smithy-go-codegen DO NOT EDIT.

package mwaa

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/mwaa/document"
	"github.com/aws/aws-sdk-go-v2/service/mwaa/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Invokes the Apache Airflow REST API on the webserver with the specified inputs.
// To learn more, see [Using the Apache Airflow REST API]
//
// [Using the Apache Airflow REST API]: https://docs.aws.amazon.com/mwaa/latest/userguide/access-mwaa-apache-airflow-rest-api.html
func (c *Client) InvokeRestApi(ctx context.Context, params *InvokeRestApiInput, optFns ...func(*Options)) (*InvokeRestApiOutput, error) {
	if params == nil {
		params = &InvokeRestApiInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "InvokeRestApi", params, optFns, c.addOperationInvokeRestApiMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*InvokeRestApiOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type InvokeRestApiInput struct {

	// The HTTP method used for making Airflow REST API calls. For example, POST .
	//
	// This member is required.
	Method types.RestApiMethod

	// The name of the Amazon MWAA environment. For example, MyMWAAEnvironment .
	//
	// This member is required.
	Name *string

	// The Apache Airflow REST API endpoint path to be called. For example,
	// /dags/123456/clearTaskInstances . For more information, see [Apache Airflow API]
	//
	// [Apache Airflow API]: https://airflow.apache.org/docs/apache-airflow/stable/stable-rest-api-ref.html
	//
	// This member is required.
	Path *string

	// The request body for the Apache Airflow REST API call, provided as a JSON
	// object.
	Body document.Interface

	// Query parameters to be included in the Apache Airflow REST API call, provided
	// as a JSON object.
	QueryParameters document.Interface

	noSmithyDocumentSerde
}

type InvokeRestApiOutput struct {

	// The response data from the Apache Airflow REST API call, provided as a JSON
	// object.
	RestApiResponse document.Interface

	// The HTTP status code returned by the Apache Airflow REST API call.
	RestApiStatusCode *int32

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationInvokeRestApiMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpInvokeRestApi{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpInvokeRestApi{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "InvokeRestApi"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addEndpointPrefix_opInvokeRestApiMiddleware(stack); err != nil {
		return err
	}
	if err = addOpInvokeRestApiValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opInvokeRestApi(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addSpanInitializeStart(stack); err != nil {
		return err
	}
	if err = addSpanInitializeEnd(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestStart(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestEnd(stack); err != nil {
		return err
	}
	return nil
}

type endpointPrefix_opInvokeRestApiMiddleware struct {
}

func (*endpointPrefix_opInvokeRestApiMiddleware) ID() string {
	return "EndpointHostPrefix"
}

func (m *endpointPrefix_opInvokeRestApiMiddleware) HandleFinalize(ctx context.Context, in middleware.FinalizeInput, next middleware.FinalizeHandler) (
	out middleware.FinalizeOutput, metadata middleware.Metadata, err error,
) {
	if smithyhttp.GetHostnameImmutable(ctx) || smithyhttp.IsEndpointHostPrefixDisabled(ctx) {
		return next.HandleFinalize(ctx, in)
	}

	req, ok := in.Request.(*smithyhttp.Request)
	if !ok {
		return out, metadata, fmt.Errorf("unknown transport type %T", in.Request)
	}

	req.URL.Host = "env." + req.URL.Host

	return next.HandleFinalize(ctx, in)
}
func addEndpointPrefix_opInvokeRestApiMiddleware(stack *middleware.Stack) error {
	return stack.Finalize.Insert(&endpointPrefix_opInvokeRestApiMiddleware{}, "ResolveEndpointV2", middleware.After)
}

func newServiceMetadataMiddleware_opInvokeRestApi(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "InvokeRestApi",
	}
}
