// Code generated by smithy-go-codegen DO NOT EDIT.

package networkflowmonitor

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/networkflowmonitor/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
	"time"
)

// Return the data for a query with the Network Flow Monitor query interface.
// Specify the query that you want to return results for by providing a query ID
// and a scope ID. This query returns data for the top contributors for workload
// insights. Workload insights provide a high level view of network flow
// performance data collected by agents for a scope.
//
// A query ID is returned from an API call to start a query of a specific type;
// for example
//
// Top contributors in Network Flow Monitor are network flows with the highest
// values for a specific metric type, related to a scope (for workload insights) or
// a monitor.
//
// The top contributor network flows overall for a specific metric type, for
// example, the number of retransmissions.
func (c *Client) StartQueryWorkloadInsightsTopContributorsData(ctx context.Context, params *StartQueryWorkloadInsightsTopContributorsDataInput, optFns ...func(*Options)) (*StartQueryWorkloadInsightsTopContributorsDataOutput, error) {
	if params == nil {
		params = &StartQueryWorkloadInsightsTopContributorsDataInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "StartQueryWorkloadInsightsTopContributorsData", params, optFns, c.addOperationStartQueryWorkloadInsightsTopContributorsDataMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*StartQueryWorkloadInsightsTopContributorsDataOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type StartQueryWorkloadInsightsTopContributorsDataInput struct {

	// The destination category for a top contributors. Destination categories can be
	// one of the following:
	//
	//   - INTRA_AZ : Top contributor network flows within a single Availability Zone
	//
	//   - INTER_AZ : Top contributor network flows between Availability Zones
	//
	//   - INTER_VPC : Top contributor network flows between VPCs
	//
	//   - AWS_SERVICES : Top contributor network flows to or from Amazon Web Services
	//   services
	//
	//   - UNCLASSIFIED : Top contributor network flows that do not have a bucket
	//   classification
	//
	// This member is required.
	DestinationCategory types.DestinationCategory

	// The timestamp that is the date and time end of the period that you want to
	// retrieve results for with your query.
	//
	// This member is required.
	EndTime *time.Time

	// The metric that you want to query top contributors for. That is, you can
	// specify this metric to return the top contributor network flows, for this type
	// of metric, for a monitor and (optionally) within a specific category, such as
	// network flows between Availability Zones.
	//
	// This member is required.
	MetricName types.WorkloadInsightsMetric

	// The identifier for the scope that includes the resources you want to get data
	// results for. A scope ID is an internally-generated identifier that includes all
	// the resources for a specific root account.
	//
	// This member is required.
	ScopeId *string

	// The timestamp that is the date and time beginning of the period that you want
	// to retrieve results for with your query.
	//
	// This member is required.
	StartTime *time.Time

	noSmithyDocumentSerde
}

type StartQueryWorkloadInsightsTopContributorsDataOutput struct {

	// The identifier for the query. A query ID is an internally-generated identifier
	// for a specific query returned from an API call to start a query.
	//
	// This member is required.
	QueryId *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationStartQueryWorkloadInsightsTopContributorsDataMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpStartQueryWorkloadInsightsTopContributorsData{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpStartQueryWorkloadInsightsTopContributorsData{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "StartQueryWorkloadInsightsTopContributorsData"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addOpStartQueryWorkloadInsightsTopContributorsDataValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opStartQueryWorkloadInsightsTopContributorsData(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addSpanInitializeStart(stack); err != nil {
		return err
	}
	if err = addSpanInitializeEnd(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestStart(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestEnd(stack); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opStartQueryWorkloadInsightsTopContributorsData(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "StartQueryWorkloadInsightsTopContributorsData",
	}
}
