// Code generated by smithy-go-codegen DO NOT EDIT.

package types

import (
	smithydocument "github.com/aws/smithy-go/document"
	"time"
)

// The key-value pair of properties for an event.
type Dimension struct {

	// The name of the dimension
	//
	// This member is required.
	Name *string

	// The value of the dimension.
	//
	// This member is required.
	Value *string

	noSmithyDocumentSerde
}

// Describes EventRule status information.
type EventRuleStatusSummary struct {

	// A human-readable reason for EventRuleStatus.
	//
	// This member is required.
	Reason *string

	// The status of the EventRule.
	//
	//   - Values:
	//
	//   - ACTIVE
	//
	//   - The EventRule can process events.
	//
	//   - INACTIVE
	//
	//   - The EventRule may be unable to process events.
	//
	//   - CREATING
	//
	//   - The EventRule is being created.
	//
	// Only GET and LIST calls can be run.
	//
	//   - UPDATING
	//
	//   - The EventRule is being updated.
	//
	// Only GET and LIST calls can be run.
	//
	//   - DELETING
	//
	//   - The EventRule is being deleted.
	//
	// Only GET and LIST calls can be run.
	//
	// This member is required.
	Status EventRuleStatus

	noSmithyDocumentSerde
}

// Contains a complete list of fields related to an EventRule.
type EventRuleStructure struct {

	// The Amazon Resource Name (ARN) of the resource.
	//
	// This member is required.
	Arn *string

	// The creation time of the resource.
	//
	// This member is required.
	CreationTime *time.Time

	// An additional event pattern used to further filter the events this EventRule
	// receives.
	//
	// For more information, see [Amazon EventBridge event patterns] in the Amazon EventBridge User Guide.
	//
	// [Amazon EventBridge event patterns]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-event-patterns.html
	//
	// This member is required.
	EventPattern *string

	// The event type to match.
	//
	// Must match one of the valid Amazon EventBridge event types. For example, EC2
	// Instance State-change Notification and AWS CloudWatch Alarm State Change. For
	// more information, see [Event delivery from AWS services]in the Amazon EventBridge User Guide.
	//
	// [Event delivery from AWS services]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-service-event.html#eb-service-event-delivery-level
	//
	// This member is required.
	EventType *string

	// A list of Amazon EventBridge Managed Rule ARNs associated with this EventRule.
	//
	// These are created by AWS User Notifications within your account so your
	// EventRules can function.
	//
	// This member is required.
	ManagedRules []string

	// The ARN for the NotificationConfiguration associated with this EventRule.
	//
	// This member is required.
	NotificationConfigurationArn *string

	// A list of AWS Regions that send events to this EventRule.
	//
	// This member is required.
	Regions []string

	// The matched event source.
	//
	// Must match one of the valid EventBridge sources. Only AWS service sourced
	// events are supported. For example, aws.ec2 and aws.cloudwatch . For more
	// information, see [Event delivery from AWS services]in the Amazon EventBridge User Guide.
	//
	// [Event delivery from AWS services]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-service-event.html#eb-service-event-delivery-level
	//
	// This member is required.
	Source *string

	// A list of an EventRule's status by Region. Regions are mapped to
	// EventRuleStatusSummary.
	//
	// This member is required.
	StatusSummaryByRegion map[string]EventRuleStatusSummary

	noSmithyDocumentSerde
}

// Describes a media element.
type MediaElement struct {

	// The caption of the media.
	//
	// This member is required.
	Caption *string

	// The unique ID for the media.
	//
	// This member is required.
	MediaId *string

	// The type of media.
	//
	// This member is required.
	Type MediaElementType

	// The url of the media.
	//
	// This member is required.
	Url *string

	noSmithyDocumentSerde
}

// Describes the components of a notification message.
type MessageComponents struct {

	// A complete summary with all possible relevant information.
	CompleteDescription *string

	// A list of properties in key-value pairs. Pairs are shown in order of importance
	// from most important to least important. Channels may limit the number of
	// dimensions shown to the notification viewer.
	//
	// Included dimensions, keys, and values are subject to change.
	Dimensions []Dimension

	// A sentence long summary. For example, titles or an email subject line.
	Headline *string

	// A paragraph long or multiple sentence summary. For example, AWS Chatbot
	// notifications.
	ParagraphSummary *string

	noSmithyDocumentSerde
}

// Contains the headline message component.
type MessageComponentsSummary struct {

	// A sentence long summary. For example, titles or an email subject line.
	//
	// This member is required.
	Headline *string

	noSmithyDocumentSerde
}

// Contains the complete list of fields for a NotificationConfiguration.
type NotificationConfigurationStructure struct {

	// The Amazon Resource Name (ARN) of the resource.
	//
	// This member is required.
	Arn *string

	// The creation time of the resource.
	//
	// This member is required.
	CreationTime *time.Time

	// The description of the NotificationConfiguration.
	//
	// This member is required.
	Description *string

	// The name of the NotificationConfiguration. Supports RFC 3986's unreserved
	// characters.
	//
	// This member is required.
	Name *string

	// The status of this NotificationConfiguration.
	//
	// The status should always be INACTIVE when part of the
	// CreateNotificationConfiguration response.
	//
	//   - Values:
	//
	//   - ACTIVE
	//
	//   - All EventRules are ACTIVE and any call can be run.
	//
	//   - PARTIALLY_ACTIVE
	//
	//   - Some EventRules are ACTIVE and some are INACTIVE .
	//
	//   - Any call can be run.
	//
	//   - INACTIVE
	//
	//   - All EventRules are INACTIVE and any call can be run.
	//
	//   - DELETING
	//
	//   - This NotificationConfiguration is being deleted. Only GET and LIST calls can
	//   be run.
	//
	//   - Only GET and LIST calls can be run.
	//
	// This member is required.
	Status NotificationConfigurationStatus

	// The aggregation preference of the NotificationConfiguration.
	//
	//   - Values:
	//
	//   - LONG
	//
	//   - Aggregate notifications for long periods of time (12 hours).
	//
	//   - SHORT
	//
	//   - Aggregate notifications for short periods of time (5 minutes).
	//
	//   - NONE
	//
	//   - Don't aggregate notifications.
	//
	// No delay in delivery.
	AggregationDuration AggregationDuration

	noSmithyDocumentSerde
}

// Describes a short summary of a NotificationEvent. This is only used when
// listing notification events.
type NotificationEventOverview struct {

	// The Amazon Resource Name (ARN) of the resource.
	//
	// This member is required.
	Arn *string

	// The creation time of the NotificationEvent.
	//
	// This member is required.
	CreationTime *time.Time

	// The ARN of the NotificationConfiguration.
	//
	// This member is required.
	NotificationConfigurationArn *string

	// Refers to a NotificationEventSummary object.
	//
	// Similar in structure to content in the GetNotificationEvent response.
	//
	// This member is required.
	NotificationEvent *NotificationEventSummary

	// The account name containing the NotificationHub.
	//
	// This member is required.
	RelatedAccount *string

	// The ARN of the aggregatedNotificationEventArn to match.
	AggregateNotificationEventArn *string

	// The NotificationConfiguration's aggregation type.
	//
	//   - Values:
	//
	//   - AGGREGATE
	//
	//   - The notification event is an aggregate notification. Aggregate
	//   notifications summarize grouped events over a specified time period.
	//
	//   - CHILD
	//
	//   - Some EventRules are ACTIVE and some are INACTIVE . Any call can be run.
	//
	//   - NONE
	//
	//   - The notification isn't aggregated.
	AggregationEventType AggregationEventType

	noSmithyDocumentSerde
}

// A NotificationEvent is a notification-focused representation of an event. They
// contain semantic information used by Channels to create end-user notifications.
type NotificationEventSchema struct {

	// The unique identifier for a NotificationEvent.
	//
	// This member is required.
	Id *string

	// A list of media elements.
	//
	// This member is required.
	Media []MediaElement

	// Describes the components of a notification message.
	//
	// This member is required.
	MessageComponents *MessageComponents

	// The type of event causing the notification.
	//
	//   - Values:
	//
	//   - ALERT
	//
	//   - A notification about an event where something was triggered, initiated,
	//   reopened, deployed, or a threshold was breached.
	//
	//   - WARNING
	//
	//   - A notification about an event where an issue is about to arise. For
	//   example, something is approaching a threshold.
	//
	//   - ANNOUNCEMENT
	//
	//   - A notification about an important event. For example, a step in a workflow
	//   or escalation path or that a workflow was updated.
	//
	//   - INFORMATIONAL
	//
	//   - A notification about informational messages. For example, recommendations,
	//   service announcements, or reminders.
	//
	// This member is required.
	NotificationType NotificationType

	// The schema version of the Notification Event.
	//
	// This member is required.
	SchemaVersion SchemaVersion

	// The source event metadata.
	//
	// This member is required.
	SourceEventMetadata *SourceEventMetadata

	// A list of text values.
	//
	// This member is required.
	TextParts map[string]TextPartValue

	// If the value of aggregationEventType is not NONE , this is the Amazon Resource
	// Event (ARN) of the parent aggregate notification.
	//
	// This is omitted if notification isn't aggregated.
	AggregateNotificationEventArn *string

	// The NotificationConfiguration's aggregation type.
	//
	//   - Values:
	//
	//   - AGGREGATE
	//
	//   - The notification event is an aggregate notification. Aggregate
	//   notifications summarize grouped events over a specified time period.
	//
	//   - CHILD
	//
	//   - Some EventRules are ACTIVE and some are INACTIVE . Any call can be run.
	//
	//   - NONE
	//
	//   - The notification isn't aggregated.
	AggregationEventType AggregationEventType

	// The end time of the event.
	EndTime *time.Time

	// The assesed nature of the event.
	//
	//   - Values:
	//
	//   - HEALTHY
	//
	//   - All EventRules are ACTIVE and any call can be run.
	//
	//   - UNHEALTHY
	//
	//   - Some EventRules are ACTIVE and some are INACTIVE . Any call can be run.
	EventStatus EventStatus

	// The source event URL.
	SourceEventDetailUrl *string

	// The detailed URL for the source event.
	SourceEventDetailUrlDisplayText *string

	// The notification event start time.
	StartTime *time.Time

	noSmithyDocumentSerde
}

// Describes a short summary and metadata for a notification event.
type NotificationEventSummary struct {

	// The notification event status.
	//
	//   - Values:
	//
	//   - HEALTHY
	//
	//   - All EventRules are ACTIVE and any call can be run.
	//
	//   - UNHEALTHY
	//
	//   - Some EventRules are ACTIVE and some are INACTIVE . Any call can be run.
	//
	// This member is required.
	EventStatus EventStatus

	// The message components of a notification event.
	//
	// This member is required.
	MessageComponents *MessageComponentsSummary

	// The type of event causing the notification.
	//
	//   - Values:
	//
	//   - ALERT
	//
	//   - A notification about an event where something was triggered, initiated,
	//   reopened, deployed, or a threshold was breached.
	//
	//   - WARNING
	//
	//   - A notification about an event where an issue is about to arise. For
	//   example, something is approaching a threshold.
	//
	//   - ANNOUNCEMENT
	//
	//   - A notification about an important event. For example, a step in a workflow
	//   or escalation path or that a workflow was updated.
	//
	//   - INFORMATIONAL
	//
	//   - A notification about informational messages. For example, recommendations,
	//   service announcements, or reminders.
	//
	// This member is required.
	NotificationType NotificationType

	// The schema version of the Notification Event.
	//
	// This member is required.
	SchemaVersion SchemaVersion

	// The source event metadata.
	//
	// This member is required.
	SourceEventMetadata *SourceEventMetadataSummary

	noSmithyDocumentSerde
}

// Describes an overview of a NotificationHub.
//
// A NotificationHub is an account-level setting used to select the Regions where
// you want to store, process and replicate your notifications.
type NotificationHubOverview struct {

	// The date and time the resource was created.
	//
	// This member is required.
	CreationTime *time.Time

	// The Region of the resource.
	//
	// This member is required.
	NotificationHubRegion *string

	// The status summary of the resource.
	//
	// This member is required.
	StatusSummary *NotificationHubStatusSummary

	// The most recent time this NotificationHub had an ACTIVE status.
	LastActivationTime *time.Time

	noSmithyDocumentSerde
}

// NotificationHub status information.
type NotificationHubStatusSummary struct {

	// An Explanation for the current status.
	//
	// This member is required.
	Reason *string

	// Status information about the NotificationHub.
	//
	//   - Values:
	//
	//   - ACTIVE
	//
	//   - Incoming NotificationEvents are replicated to this NotificationHub.
	//
	//   - REGISTERING
	//
	//   - The NotificationHub is initializing. A NotificationHub with this status
	//   can't be deregistered.
	//
	//   - DEREGISTERING
	//
	//   - The NotificationHub is being deleted. You can't register additional
	//   NotificationHubs in the same Region as a NotificationHub with this status.
	//
	// This member is required.
	Status NotificationHubStatus

	noSmithyDocumentSerde
}

// A resource affected by or closely linked to an event.
type Resource struct {

	// The Amazon Resource Name (ARN) of the resource. At least one id or ARN is
	// required.
	Arn *string

	// The URL to the resource's detail page. If a detail page URL is unavailable, it
	// is the URL to an informational page that describes the resource's type.
	DetailUrl *string

	// The unique identifier for the resource.
	//
	// At least one id or ARN is required.
	Id *string

	// A map of tags assigned to a resource. A tag is a string-to-string map of
	// key-value pairs.
	Tags []string

	noSmithyDocumentSerde
}

// Describes the metadata for a source event.
//
// For more information, see [Event structure reference] in the Amazon EventBridge User Guide.
//
// [Event structure reference]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-events-structure.html
type SourceEventMetadata struct {

	// The date and time the source event occurred. This is based on the Source Event.
	//
	// This member is required.
	EventOccurrenceTime *time.Time

	// The type of event. For example, an AWS CloudWatch state change.
	//
	// This member is required.
	EventType *string

	// The version of the type of event.
	//
	// This member is required.
	EventTypeVersion *string

	// The Primary AWS account of Source Event
	//
	// This member is required.
	RelatedAccount *string

	// A list of resources related to this NotificationEvent.
	//
	// This member is required.
	RelatedResources []Resource

	// The AWS servvice the event originates from. For example aws.cloudwatch .
	//
	// This member is required.
	Source *string

	// The source event id.
	//
	// This member is required.
	SourceEventId *string

	// The Region the event originated from.
	EventOriginRegion *string

	noSmithyDocumentSerde
}

// Contains metadata about the event that caused the NotificationEvent. For other
// specific values, see sourceEventMetadata.
type SourceEventMetadataSummary struct {

	// The event type to match.
	//
	// Must match one of the valid Amazon EventBridge event types. For example, EC2
	// Instance State-change Notification and AWS CloudWatch Alarm State Change. For
	// more information, see [Event delivery from AWS services]in the Amazon EventBridge User Guide.
	//
	// [Event delivery from AWS services]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-service-event.html#eb-service-event-delivery-level
	//
	// This member is required.
	EventType *string

	// The matched event source.
	//
	// Must match one of the valid EventBridge sources. Only AWS service sourced
	// events are supported. For example, aws.ec2 and aws.cloudwatch . For more
	// information, see [Event delivery from AWS services]in the Amazon EventBridge User Guide.
	//
	// [Event delivery from AWS services]: https://docs.aws.amazon.com/eventbridge/latest/userguide/eb-service-event.html#eb-service-event-delivery-level
	//
	// This member is required.
	Source *string

	// The Region where the notification originated.
	//
	// Unavailable for aggregated notifications.
	EventOriginRegion *string

	noSmithyDocumentSerde
}

// Describes text information objects containing fields that determine how text
// part objects are composed.
type TextPartValue struct {

	// The type of text part. Determines the usage of all other fields and whether or
	// not they're required.
	//
	// This member is required.
	Type TextPartType

	// A short single line description of the link. Must be hyperlinked with the URL
	// itself.
	//
	// Used for text parts with the type URL .
	DisplayText *string

	// A map of locales to the text in that locale.
	TextByLocale map[string]string

	// The URL itself.
	Url *string

	noSmithyDocumentSerde
}

// Stores information about a field passed inside a request that resulted in an
// exception.
type ValidationExceptionField struct {

	// A message with the reason for the validation exception error.
	//
	// This member is required.
	Message *string

	// The field name where the invalid entry was detected.
	//
	// This member is required.
	Name *string

	noSmithyDocumentSerde
}

type noSmithyDocumentSerde = smithydocument.NoSerde
