// Code generated by smithy-go-codegen DO NOT EDIT.

package securitylake

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/securitylake/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// You can use UpdateDataLake to specify where to store your security data, how it
// should be encrypted at rest and for how long. You can add a [Rollup Region]to consolidate data
// from multiple Amazon Web Services Regions, replace default encryption (SSE-S3)
// with [Customer Manged Key], or specify transition and expiration actions through storage [Lifecycle management]. The
// UpdateDataLake API works as an "upsert" operation that performs an insert if the
// specified item or record does not exist, or an update if it already exists.
// Security Lake securely stores your data at rest using Amazon Web Services
// encryption solutions. For more details, see [Data protection in Amazon Security Lake].
//
// For example, omitting the key encryptionConfiguration from a Region that is
// included in an update call that currently uses KMS will leave that Region's KMS
// key in place, but specifying encryptionConfiguration: {kmsKeyId:
// 'S3_MANAGED_KEY'} for that same Region will reset the key to S3-managed .
//
// For more details about lifecycle management and how to update retention
// settings for one or more Regions after enabling Security Lake, see the [Amazon Security Lake User Guide].
//
// [Lifecycle management]: https://docs.aws.amazon.com/security-lake/latest/userguide/lifecycle-management.html
// [Rollup Region]: https://docs.aws.amazon.com/security-lake/latest/userguide/manage-regions.html#add-rollup-region
// [Data protection in Amazon Security Lake]: https://docs.aws.amazon.com/security-lake/latest/userguide/data-protection.html
// [Amazon Security Lake User Guide]: https://docs.aws.amazon.com/security-lake/latest/userguide/lifecycle-management.html
// [Customer Manged Key]: https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#customer-cmk
func (c *Client) UpdateDataLake(ctx context.Context, params *UpdateDataLakeInput, optFns ...func(*Options)) (*UpdateDataLakeOutput, error) {
	if params == nil {
		params = &UpdateDataLakeInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "UpdateDataLake", params, optFns, c.addOperationUpdateDataLakeMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*UpdateDataLakeOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type UpdateDataLakeInput struct {

	// Specifies the Region or Regions that will contribute data to the rollup region.
	//
	// This member is required.
	Configurations []types.DataLakeConfiguration

	// The Amazon Resource Name (ARN) used to create and update the Glue table. This
	// table contains partitions generated by the ingestion and normalization of Amazon
	// Web Services log sources and custom sources.
	MetaStoreManagerRoleArn *string

	noSmithyDocumentSerde
}

type UpdateDataLakeOutput struct {

	// The created Security Lake configuration object.
	DataLakes []types.DataLakeResource

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationUpdateDataLakeMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpUpdateDataLake{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpUpdateDataLake{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "UpdateDataLake"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addOpUpdateDataLakeValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opUpdateDataLake(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addSpanInitializeStart(stack); err != nil {
		return err
	}
	if err = addSpanInitializeEnd(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestStart(stack); err != nil {
		return err
	}
	if err = addSpanBuildRequestEnd(stack); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opUpdateDataLake(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "UpdateDataLake",
	}
}
